<?php

namespace CloudPrinterPlugin\Controllers;

use CloudPrinter\CloudApps\CloudSignal\CloudSignalHandler;
use CloudPrinter\CloudApps\Exception\CloudSignalApiKeyException;
use CloudPrinterPlugin\Helpers\LogHelper;
use CloudPrinterPlugin\Helpers\OrderItemHelper;
use CloudPrinterPlugin\Libs\BaseController;
use CloudPrinterPlugin\Models\CPProduct;
use CloudPrinterPlugin\Services\DynamicProduct\DynamicProductService;

/**
 * Class CloudSignalHandlerController
 *
 * @copyright   2019 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class CloudSignalHandlerController extends BaseController
{
    /**
     * CloudSignalHandlerController constructor.
     */
    public function __construct()
    {
        // Add custom query parameter. Used for get order by reference.
        add_filter('woocommerce_order_data_store_cpt_get_orders_query', [$this, 'handleCustomQueryVar'], 10, 2);

        // Cloud signal handler endpoint.
        add_action('rest_api_init', function () {
            register_rest_route('cloudprinter/v1', '/cloud-signal/handle', [
                'methods' => 'POST',
                'callback' => [$this, 'cloudSignalHandler'],
                'permission_callback' => '__return_true'
            ]);
        });
    }

    public function cloudSignalHandler()
    {
        $signalApiKey = get_option('cp_signal_api_key');
        $cloudSignalHandler = new CloudSignalHandler($signalApiKey);

        $requestData = $cloudSignalHandler->getSignalData();
        $order = $this->getOrderByReference($requestData['order_reference']);

        if (empty($order)) {
            return new \WP_REST_Response(['message' => 'The order not found'], 404);
        }

        // Debug log.
        LogHelper::debug('debug-cloudsignal',  'Signal data:' . print_r($requestData, true));

        try {
            $cloudSignalHandler->onItemCanceled(function ($signalData) use ($order) {
                $this->itemCanceled($signalData, $order);
            });

            $cloudSignalHandler->onCloudprinterOrderValidated(function ($signalData) use ($order) {
                update_post_meta($order->get_id(), '_cp_status', $signalData['type']);
            });

            $cloudSignalHandler->onItemValidated(function ($signalData) use ($order) {
                $this->itemValidated($signalData, $order);
            });

            $cloudSignalHandler->onItemShipped(function ($signalData) use ($order) {
                $this->itemShipped($signalData, $order);
            });

            $cloudSignalHandler->onItemError(function ($signalData) use ($order) {
                $this->itemError($signalData, $order);
            });

            $cloudSignalHandler->on(['OrderCompleted'], function ($signalData)use ($order) {
                $this->orderCompleted($order);
            });

            $cloudSignalHandler->on(['CloudprinterOrderCanceled'], function ($signalData)use ($order) {
                $this->orderCanceled($order, $signalData);
            });
        } catch (CloudSignalApiKeyException $e) {
            return new \WP_REST_Response([], 403);
        }

        do_action('cloudprinter_cloud_signal_' . $requestData['type'], $requestData, $order);

        return new \WP_REST_Response(['success' => true], 200);
    }

    /**
     * Handle itemError signal
     * @param $requestData
     * @param \WC_Order $order
     */
    private function itemError($requestData, $order)
    {
        $orderId = $order->get_id();
        $orderItemId = OrderItemHelper::getOrderItemIdByCartItemKey($order, $requestData['item_reference']);
        $this->setItemStatus( $orderItemId, 'ItemError');

        // Store item error detail
        $itemErrorDetail = get_post_meta($orderId, '_cp_item_error', true);
        $itemErrorDetail = $itemErrorDetail ? json_decode($itemErrorDetail, true) : [];
        $itemErrorDetail[$requestData['item_reference']] = [
            'message' => $requestData['cause'],
            'delay' => $requestData['delay']
        ];
        update_post_meta($orderId, '_cp_item_error', json_encode($itemErrorDetail));
    }

    /**
     * Handle itemShipped signal
     * @param $requestData
     * @param $order
     */
    private function itemShipped($requestData, \WC_Order $order)
    {
        $orderItemId = OrderItemHelper::getOrderItemIdByCartItemKey($order, $requestData['item_reference']);
        $this->setItemStatus($orderItemId, 'ItemShipped');

        // Set data for order item meta
        $shippedTime = iso8601_to_datetime($requestData['datetime']);

        if (!$requestData['shipping_option']) {
            $requestData['shipping_option'] = '-';
        }

        if ($orderItemId) {
            wc_update_order_item_meta($orderItemId, '_shipping_tracking_code', $requestData['tracking']);
            wc_update_order_item_meta($orderItemId, '_shipping_tracking_url', $requestData['url']);
            wc_update_order_item_meta($orderItemId, '_shipping_option', $requestData['shipping_option']);
            wc_update_order_item_meta($orderItemId, '_order_shipped_time', $shippedTime);
        }

        $this->sendTrackingCode($order, $shippedTime);
    }

    /**
     * Handle itemCanceled signal
     * @param $requestData
     * @param \WC_Order $order
     */
    private function itemCanceled($requestData, $order)
    {
        $orderId = $order->get_id();
        $orderItemId = OrderItemHelper::getOrderItemIdByCartItemKey($order, $requestData['item_reference']);
        $this->setItemStatus($orderItemId,'ItemCanceled');
        $cancellation = get_post_meta($orderId, '_cp_items_canceled', true);
        $cancellation = $cancellation ? json_decode($cancellation, true) : [];
        $cancellation[$requestData['item_reference']] = [
            'cause' => $requestData['cause'],
            'message' => $requestData['message']
        ];
        update_post_meta($orderId, '_cp_items_canceled', json_encode($cancellation));

        // Send email
        $product = new \WC_Product($requestData['item_reference']);
        $body = 'Item was canceled by cloudprinter.com.  Order ID: ' . $orderId . '. Product: ' . $product->get_name();
        wp_mail(get_option('admin_email'), 'Item was canceled by cloudprinter.com', $body);
    }

    /**
     * Handle itemShipped signal
     */
    private function itemValidated($requestData, $order)
    {
        $orderItemId = OrderItemHelper::getOrderItemIdByCartItemKey($order, $requestData['item_reference']);
        $this->setItemStatus($orderItemId, 'ItemValidated');
    }

    /**
     * Handle orderCompleted signal
     */
    private function orderCanceled($order, $signalData)
    {
        $order->update_status( 'cancelled', 'The order has been canceled by Cloudprinter.com. Reason: ' . $signalData['message'] . '.' );
    }

    /**
     * Sand shipping tracking code to end-customers.
     * @param \WC_Order $order
     */
    private function sendTrackingCode(\WC_Order $order, $shippedTime)
    {
        $orderShippingData = $order->get_meta('_cp_shipping_data');
        $orderShippingData = $orderShippingData ? json_decode($orderShippingData, true) : [];
        $itemsShippingOptions = [];

        foreach ($orderShippingData as $orderShippingDataItem) {
            foreach ($orderShippingDataItem['items'] as $itemReference) {
                $orderItemId = OrderItemHelper::getOrderItemIdByCartItemKey($order, $itemReference);
                $itemsShippingOptions[$orderItemId] = $orderShippingDataItem['shipping_option'];
            }
        }

        $needSendShippingMail = true;
        $emailShippingData = [];

        foreach ($order->get_items() as $item) {
            $orderItemId = $item->get_id();
            $itemShippingTrackingCode = wc_get_order_item_meta($orderItemId, '_shipping_tracking_code');
            $itemShippingTrackingUrl = wc_get_order_item_meta($orderItemId, '_shipping_tracking_url');
            $itemShippingOption = wc_get_order_item_meta($orderItemId, '_shipping_option');

            if (!$itemShippingOption) {
                $needSendShippingMail = false;
                break;
            }

            $cpProduct = new CPProduct($item['product_id']);

            if (!$cpProduct->isCloudprinterProduct()) {
                continue;
            }

            $productName = $item->get_name();

            if ($cpProduct->isProductDesignerProduct() && $item['_productDesignerTemplateName']) {
                $productName .= ' |' . $item['_productDesignerTemplateName'];
            }

            $shippingName = $itemsShippingOptions[$orderItemId] ? $itemsShippingOptions[$orderItemId] : $itemShippingOption;
            $index = !empty($itemShippingTrackingCode) ? $itemShippingTrackingCode : 0;
            $emailShippingData[$shippingName][$index]['tracking_code'] = $itemShippingTrackingCode;
            if ($itemShippingTrackingUrl) {
                $emailShippingData[$shippingName][$index]['tracking_url'] = $itemShippingTrackingUrl;
            }
            $emailShippingData[$shippingName][$index]['product_name'][] = $productName;
        }

        if ($needSendShippingMail) {
            update_post_meta($order->get_id(), '_cp_order_is_shipped', 1);
            update_post_meta($order->get_id(), '_cp_order_shipped_date', $shippedTime);
            do_action('cloudprinter_order_is_shipped', $order, $emailShippingData);

            $order->update_status('completed');
        }
    }

    /**
     * Set product item status.
     * @param $itemId
     * @param $status
     */
    private function setItemStatus($itemId, $status)
    {
        wc_update_order_item_meta($itemId, '_cp_item_status', $status);
    }

    /**
     * @param $reference
     * @return \WC_Order
     */
    private function getOrderByReference($reference)
    {
        $orders = wc_get_orders([
            '_cp_order_reference' => $reference
        ]);

        return $orders[0];
    }

    /**
     * @param \WC_Order $order
     * @param $message
     */
    private function addOrderNote(\WC_Order $order, $message)
    {
        $order->add_order_note($message, true);
    }

    /**
     * Used for get order by reference
     * @param $query
     * @param $queryVars
     * @return mixed
     */
    public function handleCustomQueryVar($query, $queryVars)
    {
        if (!empty($queryVars['_cp_order_reference'])) {
            $query['meta_query'][] = array(
                'key' => '_cp_order_reference',
                'value' => esc_attr($queryVars['_cp_order_reference']),
            );
        }

        return $query;
    }
}
