<?php

namespace CloudPrinterPlugin\Controllers;

use CloudPrinterPlugin\Helpers\CloudPrinterHelper;
use CloudPrinterPlugin\Helpers\LogHelper;
use CloudPrinterPlugin\Libs\BaseController;
use CloudPrinterPlugin\Models\CPProduct;
use CloudPrinterPlugin\Services\DynamicProduct\DynamicProductService;
use CloudPrinterPlugin\Services\Storage\StorageInterface;

/**
 * Class DynamicProductController
 *
 * @copyright   2019 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class DynamicProductController extends BaseController
{
    public function __construct()
    {
        parent::__construct(CP_ABS_PATH, 'dynamic-product');

        // For products that support dynamic files.
        add_action('woocommerce_checkout_after_customer_details', [$this, 'addDynamicFileForm']);

        // Endpoint that responsible for uploading files to dropbox
        add_action('rest_api_init', function () {
            register_rest_route('cloudprinter/v1', '/dynamic-product/upload', [
                'methods' => 'POST',
                'callback' => [$this, 'uploadFile'],
                'permission_callback' => '__return_true'
            ]);
        });

        // Endpoint that responsible for removing files from dropbox
        add_action('rest_api_init', function () {
            register_rest_route('cloudprinter/v1', '/dynamic-product/remove', [
                'methods' => 'POST',
                'callback' => [$this, 'removeFile'],
                'permission_callback' => '__return_true'
            ]);
        });

        // Initialize js variables
        add_action('wp_head', [$this, 'addJsVariables']);
    }

    // Upload file to the file storage
    public function uploadFile($postData)
    {
        $dynamicProductService = new DynamicProductService();

        /** @var StorageInterface $storageService */
        $storageService = $dynamicProductService->getStorageService();

        try {
            $fileName = $storageService->uploadFileToTemporaryFolder($_FILES);

            if ($fileName) {
                // Schedule cleaning event in a case the order won`t be created.
                $time = time() + (60 * 60);
                wp_schedule_single_event($time, 'cp_clear_dynamic_product_storage', [$dynamicProductService->getStorageServiceName(), $fileName]);

                //Save temporary file uploaded data
                $data = [
                    'fileName' => $postData['fileName'],
                    'fileStoredName' => $fileName
                ];
                $itemId = $postData['itemId'];
                $fileType = $postData['fileType'];

                set_transient('user_uploaded_file_' . $itemId . '_' . $fileType, serialize($data), (60 * 60));

                return [
                    'success' => true,
                    'name' => $fileName
                ];
            }
        } catch (\Exception $e) {
            LogHelper::warning('dynamic-files', $e->getMessage());
            return ['success' => false];
        }

        return false;
    }


    // Removing file from storage
    public function removeFile($postData)
    {
        $dynamicProductService = new DynamicProductService();

        /** @var StorageInterface $storageService */
        $storageService = $dynamicProductService->getStorageService();

        try {
            $result = $storageService->removeFileFromTemporaryFolder($_POST['name']);
        } catch (\Exception $e) {
            LogHelper::warning('dynamic-files', $e->getMessage());
            return ['success' => false];
        }

        // Clear cash
        $itemId = $postData['itemId'];
        $fileType = $postData['fileType'];
        delete_transient('user_uploaded_file_' . $itemId . '_' . $fileType);

        if ($result) {
            return ['success' => true];
        }

        return false;
    }

    /**
     * Display form for dynamic file products.
     */
    public function addDynamicFileForm()
    {
        $printProducts = $this->getPrintProducts();
        $maxUploadSize = min(ini_get('post_max_size'), ini_get('upload_max_filesize'));

        $this->render('index', [
            'printProducts' => $printProducts,
            'maxUploadSize' => $maxUploadSize
        ]);
    }

    /**
     * @return array
     */
    private function getPrintProducts()
    {
        $printProducts = [];
        foreach (WC()->cart->get_cart() as $itemKey => $item) {
            /** @var \WC_Product $product */
            $product = $item['data'];
            $productModel = new CPProduct($product->get_id());
            if (CloudPrinterHelper::isCloudPrinterProduct($product->get_id()) && $productModel->isDynamicProduct()) {
                $productFilesInfo = $productModel->getCPProductFilesInfo();
                foreach ($productFilesInfo as &$productFileInfo) {
                    $productFileInfo['uploaded_file'] = null;
                    $uploadedFileData = get_transient('user_uploaded_file_' . $itemKey . '_' . $productFileInfo['type']);
                    if (!empty($uploadedFileData)) {
                        $productFileInfo['uploaded_file'] = unserialize($uploadedFileData);
                    }
                }

                $printProducts[] = [
                    'item_id' => $itemKey,
                    'name' => $product->get_name(),
                    'files' => $productFilesInfo,
                    'count_files_in_archive' => $productModel->getProductOptionValue('type_archive_total_files'),
                    'uploaded_file' => false
                ];


            }
        }

        return $printProducts;
    }

    /**
     * Initialize js variables
     */
    public function addJsVariables()
    {
        $maxUploadSize = min(ini_get('post_max_size'), ini_get('upload_max_filesize'));
        $maxUploadSizeBytes = (int)$maxUploadSize * 1024 * 1024;
        $maxUploadSizeMB = str_replace('M', 'MB', $maxUploadSize);
        echo "
            <script>
                var maxFileSizeBytes = '" . $maxUploadSizeBytes . "';
                var maxFileSize = 'The max file size is " . $maxUploadSizeMB . "';
            </script>
        ";
    }
}
