<?php

namespace CloudPrinterPlugin\Controllers;

use CloudPrinter\CloudApps\Authentication\OAuth2;
use CloudPrinterPlugin\Helpers\CloudPrinterHelper;
use CloudPrinterPlugin\Helpers\ConfigHelper;
use CloudPrinterPlugin\Helpers\LogHelper;
use CloudPrinterPlugin\Libs\BaseController;
use CloudPrinterPlugin\Services\Storage\DropboxService;

class GeneralSettingsController extends BaseController
{
    public function __construct()
    {
        parent::__construct(CP_ABS_PATH, 'general-settings');
        add_action('admin_menu', [$this, 'addAdminMenuItem']);
        add_action('admin_init', [$this, 'adminInit']);
    }

    public function addAdminMenuItem()
    {
        add_options_page(
            'Cloudprinter.com',
            'Cloudprinter.com',
            'manage_options',
            'cloudprinter',
            [$this, 'renderOptionsPage']
        );
    }

    public function adminInit()
    {
        add_settings_section(
            'cloudprinter_section',
            '',
            [$this, 'renderSection'],
            'cloudprinter'
        );

        $config = ConfigHelper::getParam('authorization');
        $oAuth2 = new OAuth2($config);
        $accessToken = get_option('cp_api_key');

        // Cloudprinter logout handling
        if (!empty($_GET['cp_logout'])) {
            delete_option('cp_api_key');
            delete_option('cp_authorized_user_login');
            $accessToken = null;
            wp_redirect(get_site_url() . '/wp-admin/options-general.php?page=cloudprinter');
        }
        // Dropbox logout handling
        if (!empty($_GET['dropbox_logout'])) {
            delete_option('cp_dropbox_access_token');
            delete_option('cp_dropbox_username');
            wp_redirect(get_site_url() . '/wp-admin/options-general.php?page=cloudprinter');
        }

        // Getting access token
        if (isset($_GET['code'])) {
            $response = $oAuth2->getAccessToken($_GET['code']);
            $responseData = $response->getData();
            $accessToken = $responseData['access_token'] ?? '';
            update_option('cp_api_key', $accessToken);
            CloudPrinterHelper::webHookSubscribe();

            wp_redirect(get_site_url() . '/wp-admin/options-general.php?page=cloudprinter');
        }

        // Store authorized user login
        $userLogin = get_option('cp_authorized_user_login');
        if ($accessToken && !$userLogin) {
            $userInfo = CloudPrinterHelper::getUserInfo();
            update_option('cp_authorized_user_login', $userInfo['organization']['name']);
        }

        $authorizationUrl = $oAuth2->getAuthorizationCodeUrl();

        // Save general settings.
        register_setting('cloudprinter', 'cp_api_mode', [$this, 'check_by_for_all']);
        register_setting('cloudprinter', 'cp_order_submission_type', [$this, 'check_by_for_all']);
        register_setting('cloudprinter', 'cp_dynamic_product_storage', [$this, 'check_by_for_all']);
        if (isset($_POST['cp_dropbox_access_token'])) {
            register_setting('cloudprinter', 'cp_dropbox_access_token', [ $this, 'check_by_for_all' ]);
        }
        register_setting('cloudprinter', 'cp_cost_check', [$this, 'check_by_for_all']);
        register_setting('cloudprinter', 'cp_free_shipping', [$this, 'check_by_for_all']);
        register_setting('cloudprinter', 'cp_min_product_price_margin', [$this, 'check_by_for_all']);
        register_setting('cloudprinter', 'cp_product_price_margin_alert_percentage', [$this, 'check_by_for_all']);
        register_setting('cloudprinter', 'cp_support_email', [$this, 'check_by_for_all']);
        register_setting('cloudprinter', 'cp_shipping_price_margin', [$this, 'check_by_for_all']);
        register_setting('cloudprinter', 'cp_include_sales_cost', [$this, 'check_by_for_all']);
        register_setting('cloudprinter', 'cp_debug_log', [$this, 'check_by_for_all']);

        $dropboxAccessToken = get_option('cp_dropbox_access_token');

        if (!empty($_POST['cp_dropbox_access_token']) && $_POST['cp_dropbox_access_token'] !== $dropboxAccessToken) {
            $dropboxService = new DropboxService($_POST['cp_dropbox_access_token']);
            try {
                $userName = $dropboxService->getCurrentUsername();

                if ($userName) {
                    update_option('cp_dropbox_username', $userName);
                    delete_option('cp_dropbox_access_token_error');
                } else {
                    delete_option('cp_dropbox_username');
                    update_option('cp_dropbox_access_token_error', 'The access token is not valid');
                }

            } catch (\Exception $e) {

                delete_option('cp_dropbox_username');
                update_option('cp_dropbox_access_token_error', 'The access token is not valid');
                LogHelper::warning('dynamic-files', $e->getMessage());
            }
        }

        $this->render('oauth2', [
            'accessToken' => $accessToken,
            'authorizationUrl' => $authorizationUrl
        ]);

        $this->render('api_mode');
        $this->render('order_submission_type');
        $this->render('support_email');
        $this->render('cost_check');
        $this->render('min_product_price_margin');
        $this->render('product_price_margin_alert_percentage');
        $this->render('free_shipping');
        $this->render('shipping_price_margin');
        $this->render('include_sales_cost');
        $this->render('debug_log');
        $this->render('dynamic_product_storage');
        $this->render('dropbox_access_token', ['dropboxAccessToken' => $dropboxAccessToken]);
    }

    public function renderOptionsPage()
    {
        $this->render('index');
    }

    public function renderSection()
    {
        return '';
    }
}
