<?php

namespace CloudPrinterPlugin\Controllers;

use CloudPrinterPlugin\Helpers\CloudPrinterHelper;
use CloudPrinterPlugin\Helpers\LogHelper;
use CloudPrinterPlugin\Libs\BaseController;
use CloudPrinterPlugin\Models\CPProduct;
use CloudPrinterPlugin\Services\Storage\LocalService;
use CloudPrinterPlugin\Services\Storage\StorageInterface;

/**
 * Class DynamicProductController
 *
 * @copyright   2019 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class ManualProductFileUploadController extends BaseController
{
    public function __construct()
    {
        parent::__construct(CP_ABS_PATH, 'manual-product-file-uplaod');

        // For products that support dynamic files.
        add_action('woocommerce_after_order_itemmeta', [$this, 'addOrderItemFileForm'], 10, 3);
        add_action('woocommerce_update_order_item', [$this, 'saveOrderItemCustomFields'], 10, 3);

        // Endpoint that responsible for uploading files to dropbox
        add_action('rest_api_init', function () {
            register_rest_route('cloudprinter/v1', '/admin/dynamic-product/upload', [
                'methods' => 'POST',
                'callback' => [$this, 'uploadFile'],
                'permission_callback' => '__return_true'
            ]);
        });


        // Endpoint that responsible for removing files from dropbox
        add_action('rest_api_init', function () {
            register_rest_route('cloudprinter/v1', '/admin/dynamic-product/remove', [
                'methods' => 'POST',
                'callback' => [$this, 'removeFile'],
                'permission_callback' => '__return_true'
            ]);
        });

        // Initialize js variables
        add_action('admin_head', [$this, 'addJsVariables']);
    }

    public function saveOrderItemCustomFields($itemId, $item)
    {
        if ($_POST['items']) {
            $postData = [];
            parse_str($_POST['items'], $postData);

            if ($postData['cp_product_file_name']) {
                $item->update_meta_data('_cp_product_files_name', $postData['cp_product_file_name']);
            }

            if ($postData['cp_product_file_url']) {
                $item->update_meta_data('_cp_product_files_url', $postData['cp_product_file_url']);
            }

            if ($postData['cp_file_storage']) {
                $item->update_meta_data('_cp_product_files_storages', $postData['cp_file_storage']);
            }
        }
    }

    // Upload file to the file storage
    public function uploadFile($postData)
    {
        /** @var StorageInterface $storageService */
        $storageService = new LocalService();

        try {
            $fileName = $storageService->uploadFileToOrderedFolder($_FILES);

            if ($fileName) {
                return [
                    'success' => true,
                    'name' => $fileName
                ];
            }
        } catch (\Exception $e) {
            LogHelper::warning('manual-file-upload', $e->getMessage());
            return ['success' => false];
        }

        return false;
    }


    // Removing file from storage
    public function removeFile($postData)
    {
        /** @var StorageInterface $storageService */
        $storageService = new LocalService();

        try {
            $result = $storageService->removeFilesFromOrderedFolder($_POST['name']);
        } catch (\Exception $e) {
            LogHelper::warning('manual-file-upload', $e->getMessage());
            return ['success' => false];
        }

        if ($result) {
            return ['success' => true];
        }

        return false;
    }

    /**
     * Display form for dynamic file products.
     */
    public function addOrderItemFileForm($itemId, $item, $product)
    {
        // For shipping lineItem the product is empty.
        if (!$product) {
            return true;
        }

        $lineItemDetails = $this->getOrderLineItemDetails($itemId, $item, $product);
        $maxUploadSize = min(ini_get('post_max_size'), ini_get('upload_max_filesize'));
        $this->render('index', [
            'itemId' => $itemId,
            'lineItemDetails' => $lineItemDetails,
            'maxUploadSize' => $maxUploadSize
        ]);
    }
    /**
     * @return array
     */
    private function getOrderLineItemDetails($itemKey, $item, $product)
    {
        $printProduct = null;
        $storageService = new LocalService();

        $uploadedFilesNames = $item->get_meta('_cp_product_files_name');
        $filesLink = $item->get_meta('_cp_product_files_url');
        $storage = $item->get_meta('_cp_product_files_storages');
        $productModel = new CPProduct($product->get_id());
        if (CloudPrinterHelper::isCloudPrinterProduct($product->get_id()) && $productModel->isManualFileUploadSupported()) {
            $productFilesInfo = $productModel->getCPProductFilesInfo();

            foreach ($productFilesInfo as &$productFileInfo) {


                if ($uploadedFilesNames && $uploadedFilesNames[$itemKey][$productFileInfo['type']]) {
                    $productFileInfo['uploadedFileName'] = $uploadedFilesNames[$itemKey][$productFileInfo['type']];
                    $productFileInfo['uploadedFileLink'] = $storageService->getSharedLink($productFileInfo['uploadedFileName']);
                }

                if ($filesLink && $filesLink[$itemKey][$productFileInfo['type']]) {
                    $productFileInfo['fileLink'] = $filesLink[$itemKey][$productFileInfo['type']];
                }
            }

            $printProduct = [
                'item_id' => $itemKey,
                'storage' => $storage[$itemKey],
                'files' => $productFilesInfo,
                'count_files_in_archive' => $productModel->getProductOptionValue('type_archive_total_files'),
                'uploaded_file' => false
            ];
        }
        return $printProduct;
    }

    /**
     * Initialize js variables
     */
    public function addJsVariables()
    {
        $maxUploadSize = min(ini_get('post_max_size'), ini_get('upload_max_filesize'));
        $maxUploadSizeBytes = (int)$maxUploadSize * 1024 * 1024;
        $maxUploadSizeMB = str_replace('M', 'MB', $maxUploadSize);
        echo "
            <script>
                var maxFileSizeBytes = '" . $maxUploadSizeBytes . "';
                var maxFileSize = 'The max file size is " . $maxUploadSizeMB . "';
            </script>
        ";
    }
}
