<?php

namespace CloudPrinterPlugin\Controllers;

use CloudPrinterPlugin\Helpers\{ArrayHelper, CloudPrinterHelper, OrderHelper};
use CloudPrinterPlugin\Libs\BaseController;
use CloudPrinterPlugin\Models\CPOrder;

/**
 * Class OrderDetailController
 *
 * @copyright   2019 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class OrderDetailController extends BaseController
{
    public function __construct()
    {
        parent::__construct(CP_ABS_PATH, 'order-detail');

        // Prepare list of stock and print shipping
        add_filter('woocommerce_get_order_item_totals', [$this, 'displayShipping'], 10, 2);

        add_action('woocommerce_order_item_meta_end', [$this, 'displayItemMeta'], 10, 3);

   //     add_filter('woocommerce_order_get_total_tax', [$this, 'cloudprinterCalcTotalTax'], 10, 2);
    }

    public function cloudprinterCalcTotalTax($tax, $order)
    {
        $order = new CPOrder($order->get_id());

        return $order->get_shipping_tax();
    }

    public function displayItemMeta($item_id, $item, $order)
    {
        $dynamicOptions = $item->get_meta('_dynamicOptions');
        if (!empty($dynamicOptions)) {
            $viewData = ['productDynamicOptions' => CloudPrinterHelper::getItemDynamicOption($item['product_id'], unserialize($dynamicOptions))];
            $this->render('item-meta', $viewData);
        }
    }


    /**
     * @param $orderItems
     * @return mixed
     */
    private function getPrintProduct($orderItems)
    {
        foreach ($orderItems as $key => $orderItem) {
            if (!CloudPrinterHelper::isCloudPrinterProduct($orderItem['product_id'])) {
                unset($orderItems[$key]);
            }
        }

        return $orderItems;
    }

    /**
     * @param $totalRows
     * @param $order \WC_Order
     * @return array
     */
    public function displayShipping($totalRows, $order)
    {
        $freeShipping = $order->get_meta('_cp_shipping_is_free') || OrderHelper::hasFreeShippingCoupon($order) ;
        $shippingMarginPercentage = $order->get_meta('_cp_shipping_margin_percentage') ?: 0;

        $items = $this->getPrintProduct($order->get_items());

        if (!$items) {
            return $totalRows;
        }

        // Prepare list of products that included in current shipping.
        $products = [];
        foreach ($this->getStockProductIds($order->get_items()) as $stockProductId) {
            $products[] = $this->getProductNameById($order->get_items(), $stockProductId);
        }

        // Changer label for stock shipping.
        foreach ($totalRows as $key => &$totalRow) {
            if ($key == 'shipping') {
                $totalRow['label'] = 'Stock shipping:';
                $totalRow['value'] .= "<br><small><b>Products:</b> " . join(', ', $products) . "</small>";
            }
        }

        // Shipping data of order from session
        $shippingData = get_post_meta($order->get_id(), '_cp_shipping_data', true);
        $shippingData = $shippingData ? json_decode($shippingData, true) : [];

        //Display shipping for print products.
        foreach ($shippingData as $key => $shipping) {
            if ($freeShipping) {
                $shippingPrice = 'Free';
            } else {
                $priceInclMargin = CloudPrinterHelper::calcPercentage($shipping['price'], $shippingMarginPercentage);
                $shippingPrice = wc_price($priceInclMargin);
            }

            $shippingName = sprintf('%s (%s)', $shipping['service'], $shipping['shipping_option']);
            $shippingNameToDisplay = apply_filters('cloudprinter_shipping_name', $shippingName, $shipping['service'], $shipping['shipping_option']);

            // Prepare list of products that included in current shipping.
            $products = [];
            foreach ($shipping['items'] as $item) {
                $products[] = $this->getProductNameById($items, $item);
            }

            $productsHtml = "<br><small><b>Products:</b> " . join(', ', $products) . "</small>";
            $printShipping = [
                'label' => apply_filters('cp_order_print_shipping_label', 'Print shipping:'),
                'value' => sprintf('%s<small class="shipped_via"> via %s</small>' . $productsHtml, $shippingPrice, $shippingNameToDisplay)
            ];

            // Insert print shipping in order details.
            if (isset($totalRow['shipping'])) {
                $totalRows = ArrayHelper::array_insert(
                    $totalRows,
                    'shipping',
                    'print_shipping' . $key,
                    $printShipping
                );
            } else {
                $totalRows = ArrayHelper::array_insert(
                    $totalRows,
                    'cart_subtotal',
                    'print_shipping' . $key,
                    $printShipping
                );
            }
        }

        return $totalRows;
    }

    /**
     * @param $items
     * @param $productId
     * @return string
     */
    private function getProductNameById($items, $key)
    {
        $productName = null;

        foreach ($items as $item) {
            /** @var \WC_Product $product */
            $product = $item->get_product();



            if ($item->get_meta('_cartItemKey') == $key) {
                $productName = $product->get_name();
                break;
            }
        }

        return $productName;
    }

    /**
     * @param $items
     * @return array
     */
    private function getStockProductIds($items)
    {
        $stockProductIds = [];
        foreach ($items as $item) {
            /** @var \WC_Product $product */
            $product = $item->get_product();
            if (!CloudPrinterHelper::isCloudPrinterProduct($product->get_id())) {
                $stockProductIds[] = $item->get_meta('_cartItemKey');
            }
        }

        return $stockProductIds;
    }
}
