<?php

namespace CloudPrinterPlugin\Controllers;

use CloudPrinter\CloudApps\Client\CloudAppsClient;
use CloudPrinter\CloudApps\Exception\ValidationException;
use CloudPrinter\CloudApps\Model\{Address, File, Option, Order, OrderItem};
use CloudPrinterPlugin\Helpers\{CartHelper, CloudPrinterHelper, LogHelper, OrderHelper};
use CloudPrinterPlugin\Models\CPOrder;
use CloudPrinterPlugin\Models\CPProduct;
use CloudPrinterPlugin\Services\ConfigService;
use CloudPrinterPlugin\Services\DynamicProduct\DynamicProductService;
use CloudPrinterPlugin\Services\ShippingService;
use CloudPrinterPlugin\Services\Storage\LocalService;
use mysql_xdevapi\Exception;

/**
 * Class OrderPostingController
 *
 * @copyright   2019 by CloudPrinter
 * @author      Vasyl Kushniruk <vk@cloudprinter.com>
 */
class OrderPostingController
{
    private $accessToken;

    private $configService;

    public function __construct()
    {
        $this->configService = new ConfigService();
        $this->accessToken = $this->configService->getCPAccessToken();

        add_action('woocommerce_new_order', [$this, 'newOrderCreated']);

        // Preparing for order submission to cp
        add_action('woocommerce_order_status_processing', [$this, 'orderIsPaid']);

        // Start order submission process.
        add_action('cloudprinter_post_order', [$this, 'startOrderSubmissionProcess']);

        // Manually order submission from admin
        add_action('rest_api_init', function () {
            register_rest_route('cloudprinter/v1', '/order/post', [
                'methods' => 'POST',
                'callback' => [$this, 'postOrderAPIAction'],
                'permission_callback' => [$this, 'verifyApiRequest']
            ]);
        });
    }

    /**
     * Post order to cloudprinter from API endpoint.
     * Only for admin
     * @param $data
     * @return array|bool[]
     */
    public function postOrderAPIAction($data)
    {
        $orderId = $data['order_id'];
        $isReorder = false;
        $order = new \WC_Order($orderId);

        do_action('cloudprinter_before_order_posting', $order, $isReorder);

        $itemQuotes = $data['item_quotes'];
        if (!$itemQuotes) {
            $itemQuotes = $this->getOrderNativeQuotes($order, $isReorder);
        }

        $result = $this->postOrder($order, $isReorder, $itemQuotes);

        if (true === $result) {
            return [
                'success' => true,
            ];
        } else {
            return [
                'success' => false,
                'error' => $result
            ];
        }
    }

    // Store needed data after creation the order.
    public function newOrderCreated($orderId)
    {
        $order = new CPOrder($orderId);

        // Break, if reorder
        if ($isReorder = $order->get_meta('_cp_reorder')) {
            return true;
        }

        // Save thr shipping price margin;
        $shippingMarginPercentage = $this->configService->getShippingPriceMargin();
        update_post_meta($orderId, '_cp_shipping_margin_percentage', $shippingMarginPercentage);


        //Save, if free shipping
        $isFreeShipping = $this->configService->isFreeShipping();
        if ($isFreeShipping) {
            update_post_meta($orderId, '_cp_shipping_is_free', true);
        }

        // Save the shipping data.
        $wcSession = CloudPrinterHelper::getWCSession();
        $shippingData = $wcSession->get('cp_shipping_data');
        if ($shippingData) {
            update_post_meta($orderId, '_cp_shipping_data', $shippingData);
            $wcSession->set('cp_shipping_data', '');

            if (!$isFreeShipping) {
                $shippingService = new ShippingService();
                $totalShippingTax = $order->get_shipping_tax();
                $totalShipping = 0;
                $shippingLines = json_decode($shippingData, true);
                if (is_array($shippingLines)) {
                    foreach ($shippingLines as $shippingLine) {
                        if (wc_tax_enabled()) {
                            $shippingLinePrice = isset($shippingLine['native_price']) ? $shippingLine['native_price'] : $shippingLine['price'];
                            $price = CloudPrinterHelper::calcPercentage($shippingLinePrice, $shippingMarginPercentage);
                            $totalShippingTax += $shippingService->calculateTax($price);
                        }

                        $totalShipping += $shippingLine['price'];
                    }
                }

                // Store the shipping cost & tax
                if (!CartHelper::cartHasFreeShippingCoupon()) {
                    $order->update_meta_data('_cp_order_shipping', $totalShipping);
                    $order->update_meta_data('_cp_order_shipping_tax', $totalShippingTax);

                    $order->set_shipping_tax($totalShippingTax);
                    $order->save();
                } else {
                    update_post_meta($order->ID, '_cart_discount', $totalShipping);
                }
            }
        } else {
            LogHelper::warning('warning-order-quotes','The quotes not found in session. Order id:' . $orderId);
        }

        //Save the cloudprinter product price
        $purchasePrice = $wcSession->get('cp_product_price');
        if (!empty($purchasePrice)) {
            update_post_meta($orderId, '_cp_order_total_product_price', $purchasePrice);
        }
    }

    /**
     * Start order submission process.
     */
    public function orderIsPaid($orderId) {
        if ($this->configService->isSubmissionImmediately()) {
            $this->startOrderSubmissionProcess($orderId);
        } elseif ($this->configService->isSubmissionInBackground()) {
            wp_schedule_single_event(time() + 60,'cloudprinter_post_order', [$orderId]);
        }
    }

    /**
     * Post order after payment.
     */
    public function startOrderSubmissionProcess($orderId)
    {
        $order = new \WC_Order($orderId);
        $isReorder = get_post_meta($orderId, '_cp_reorder', true);

        if(!apply_filters('cloudprinter_order_is_ready_for_posting', true, $orderId)) {
            return false;
        }

        do_action('cloudprinter_before_order_posting', $order, $isReorder);
        $itemQuotes = null;
        if (!$isReorder) {
            $itemQuotes = $this->getOrderNativeQuotes($order);

            if (!$itemQuotes) {
                LogHelper::warning('warning-order-quotes', 'Quotes are empty after payment. Order id: ' . $orderId . '.');
                throw new \Exception('Shipping is not available now.');
            }
        }

        return $this->postOrder($order, $isReorder, $itemQuotes);
    }

    // Get the quotes that were selected by customer.
    private function getOrderNativeQuotes(\WC_Order $order)
    {
        $orderId = $order->get_id();

        // Order quote
        $shippingData = $order->get_meta('_cp_shipping_data');
        $shippingData = json_decode($shippingData, true);
        $nativeQuotes = [];

        if ($shippingData) {
            foreach ($shippingData as $shippingDataItem) {
                foreach ($shippingDataItem['items'] as $item) {
                    $nativeQuotes[$item] = $shippingDataItem['quote'];
                }
            }
        }

        if (!$nativeQuotes) {
            try {
                $itemQuotes = $this->getOrderQuotes($order);
                update_post_meta($orderId, '_cp_shipping_data', json_encode($itemQuotes));
            } catch (\Exception $e) {
                update_post_meta($orderId, '_cp_status', 'error');
                update_post_meta($orderId, '_cp_error_message', 'Get order quotes. ' . $e->getMessage());
                return false;
            }
        } else {
            $itemQuotes = $nativeQuotes;
        }

        return $itemQuotes;
    }

    private function postOrder(\WC_Order $order, $isReorder, $itemQuotes)
    {
        // Forbid to post order twice to cloudprinter.
        if (!$isReorder && !empty($order->get_meta('_cp_order_reference'))) {
            return true;
        }

        $orderId = $order->get_id();
        $printProducts = CloudPrinterHelper::getPrintProducts($order);

        // Dynamic file product
        try {
            $dynamicProductService = new DynamicProductService();
            $dynamicProductService->requestSharedLinksForOrder($order);
            $dynamicProductData = json_decode(get_post_meta($orderId,'_cp_dynamic_product', true), true);
        } catch (\Exception $e) {
            LogHelper::warning('dynamic-files', $e->getMessage());
            do_action('cloudprinter_order_posting_error', $orderId, 200, $e->getMessage());
            return false;
        }
        if (!$printProducts) {
            return true;
        }

        $orderReference = 'wp-' . $orderId . '-' . md5($orderId . $this->accessToken);
        $address = $this->getAddress($order);
        $cloudPrinterClient = new CloudAppsClient($this->accessToken);
        $apiMode = $this->configService->getOrderMode();

        // Collect cloudprinter order data
        $cpOrder = new Order();
        $cpOrder->setReference($orderReference);
        $cpOrder->setMode($apiMode);
        $cpOrder->setEmail($this->configService->getAdminEmail());
        $cpOrder->addAddress($address);

        try {
            // Prepare order items
            foreach ($printProducts as $key => $printProduct) {
                if (!$printProduct['product_reference']) {
                    $errorMessage = sprintf('The product reference is not set for the product_id: %s', $printProduct['product_id']) ;
                    update_post_meta($orderId, '_cp_status', 'error');
                    update_post_meta($orderId, '_cp_error_message', $errorMessage);
                    do_action('cloudprinter_product_reference_is_missing', $printProduct['product_id']);
                    do_action('cloudprinter_order_posting_error', $orderId, 0, $errorMessage);
                    return false;
                }

                $productId = $printProduct['product_id'];
                $productModel = new CPProduct($productId);
                $productPeaces = CloudPrinterHelper::getPeacesForProduct($productId);
                $cpOrderItem = new OrderItem();

                // Set additional data for reordering
                if ($isReorder) {
                    $reorderReference = get_post_meta($orderId, '_cp_order_reference', true);

                    $cpOrderItem->setReorderItemReference($printProduct['item_key'])
                        ->setReorderOrderReference($reorderReference)
                        ->setReorderCause($printProduct['reorder_cause'])
                        ->setReorderDescription($printProduct['reorder_description']);
                }

                $cpOrderItem->setReference($printProduct['item_key']);
                $cpOrderItem->setProductReference($printProduct['product_reference']);

                if (!$isReorder) {
                    if (!empty($itemQuotes[$printProduct['item_key']])) {
                        $cpOrderItem->setQuote($itemQuotes[$printProduct['item_key']]);
                    } else {
                        $errorMessage = 'Order_item_key: ' . $printProduct['item_key'] . '. Order quotes: ' . print_r($itemQuotes, true);
                        LogHelper::warning('warning-order-quotes', $errorMessage);
                    }
                }

                $cpOrderItem->setCount($printProduct['product_quantity'] * $productPeaces);

                // Prepare product files
                if ($productModel->isProductDesignerProduct()) {
                    $productDesigner = $productModel->getProductDesigner();
                    $fileUrl = $productDesigner->getProjectPdfUrl($printProduct['orderItemId']);
                    if ($fileUrl) {
                        $file = new File();
                        $file->setUrl($fileUrl);
                        $file->setType('product');
                        $file->setMd5sum('_____________NO_MD5_____________');
                        $cpOrderItem->addFile($file);
                    }
                } else if ($productModel->isDynamicProduct()) {
                    foreach ($dynamicProductData[$printProduct['item_key']] as $dynamicProductFile) {
                        $file = new File();

                        if ($dynamicProductFile['storage'] == 'local') {
                            $file->setUrl($dynamicProductFile['shared_link']);
                        } else {
                            $file->setUrl($dynamicProductFile['file']);
                        }

                        $file->setType($dynamicProductFile['type']);
                        $file->setMd5sum('_____________NO_MD5_____________');
                        $cpOrderItem->addFile($file);
                    }
                } else if ($productModel->isManualFileUploadSupported()) {
                    $storageService = new LocalService();
                    $fileStorage = $printProduct['lineItem']->get_meta('_cp_product_files_storages');
                    $externalFiles = $printProduct['lineItem']->get_meta('_cp_product_files_url');
                    $localFiles = $printProduct['lineItem']->get_meta('_cp_product_files_name');
                    $itemId = $printProduct['orderItemId'];
                    foreach ($printProduct['filesInfo'] as $productFileInfo) {
                        $file = new File();
                        $cpFileUrl = null;

                        if ($fileStorage[$itemId] == 'local' && $localFiles[$itemId][$productFileInfo['type']]) {
                            $cpFileUrl = $storageService->getSharedLink($localFiles[$itemId][$productFileInfo['type']]);
                        } else if ($fileStorage[$itemId] == 'external'){
                            $cpFileUrl = $externalFiles[$itemId][$productFileInfo['type']];
                        }

                        if (!$cpFileUrl) {
                            throw new \Exception('The file is not configured for one of the products.');
                        }

                        $file->setUrl($cpFileUrl);
                        $file->setType($productFileInfo['type']);
                        $file->setMd5sum('_____________NO_MD5_____________');
                        $cpOrderItem->addFile($file);
                    }
                } else {
                    foreach ($printProduct['files'] as $fileType => $fileData) {
                        $fileUrl = apply_filters('cloudprinter_get_orders_line_item_file_url', $fileData['url'], $order, $printProduct['orderItemId'], $fileType);

                        // Reset md5 hash if the file was overridden
                        if ($fileUrl !== $fileData['url']) {
                            $fileData['md5'] = false;
                        }

                        $fileMd5 = apply_filters('cloudprinter_get_orders_line_item_file_md5', $fileData['md5'], $order, $printProduct['orderItemId'], $fileType);
                        $file = new File();
                        $file->setUrl($fileUrl);
                        $file->setType($fileType);
                        if (!empty($fileData['md5'])) {
                            $file->setMd5sum($fileMd5);
                        }

                        $cpOrderItem->addFile($file);
                    }
                }

                // Prepare product options
                foreach ($printProduct['product_options'] as $key => $productOption) {
                    $option = new Option();

                    if ($key == 'type_archive_total_files') {
                        $option->setOptionReference('archive_total_files');
                        $option->setCount($productOption);
                    } else {
                        $option->setOptionReference($productOption);
                        $option->setCount(1);
                    }

                    $cpOrderItem->addOption($option);
                }

                // Set total_page option for book products
                if (CloudPrinterHelper::isBookProduct($printProduct['product_category'])) {
                    $option = new Option();
                    $option->setOptionReference('total_pages');
                    $option->setCount(get_post_meta($productId, 'cp_book_count_pages', true));
                    $cpOrderItem->addOption($option);
                }

                $cpOrderItem = apply_filters('cloudprinter_before_order_item_add', $cpOrderItem,  $order, $printProduct['orderItemId']);
                $cpOrder->addItem($cpOrderItem);
            }


            $cpOrder = apply_filters('cloudprinter_before_order_submit', $cpOrder, $order);
            update_post_meta($orderId, '_cp_order_json', json_encode($cpOrder->toArray()));
            $response = $cloudPrinterClient->order->create($cpOrder);
            $responseData = $response->getData();

            // Write log
            $message = 'orders/add | Request:' . json_encode($cpOrder->toArray()) . PHP_EOL . 'Response:' . json_encode($response->getData());
            LogHelper::debug('debug-api', $message);
            update_post_meta($orderId, '_cp_order_mode', $apiMode);
            if (201 == $response->getStatusCode()) {
                $data = $response->getData();
                update_post_meta($orderId, '_cp_status', $isReorder ? 'reordered' : 'posted');
                update_post_meta($orderId, '_cp_error_message', '');
                update_post_meta($orderId, '_cp_order_reference', $data['reference']);
                do_action('cloudprinter_order_posting_success', $orderId, $apiMode, $data['reference']);

                return true;
            } elseif (is_array($responseData) && isset($responseData['error']['info'])) {
                update_post_meta($orderId, '_cp_status', 'error');
                update_post_meta($orderId, '_cp_error_message', $responseData['error']['info']);
                do_action('cloudprinter_order_posting_error', $orderId, $response->getStatusCode(), $responseData['error']['info']);

                return $responseData['error']['info'];
            }
        } catch (ValidationException $e) {
            update_post_meta($orderId, '_cp_status', 'error');
            update_post_meta($orderId, '_cp_error_message', print_r($e->getValidationMessages(), true));
            do_action('cloudprinter_order_posting_error', $orderId, 200, $e->getValidationMessages());

            return $this->getFirstValidationMessage($e);
        } catch (\Exception $e) {
            update_post_meta($orderId, '_cp_status', 'error');
            update_post_meta($orderId, '_cp_error_message', print_r($e->getMessage(), true));
            do_action('cloudprinter_order_posting_error', $orderId, 200, $e->getMessage());

            return $e->getMessage();
        }
    }

    /**
     * Get first error message.
     * @param ValidationException $e
     * @return mixed|string|void
     */
    private function getFirstValidationMessage(ValidationException $e)
    {
        $validationFieldMessages = $e->getValidationMessages();
        if (is_string($validationFieldMessages)) {
            return $validationFieldMessages;
        }

        $validationFieldMessagesValues = array_values($validationFieldMessages);
        $firstFieldMessages = array_shift($validationFieldMessagesValues);

        if (is_array($firstFieldMessages)) {
            $fieldsMessageValues = array_values($firstFieldMessages);
            return  ucfirst(array_shift($fieldsMessageValues));
        } else {
            return $firstFieldMessages;
        }
    }

    /**
     * Prepare address for order
     * @param \WC_Order $order
     * @return Address
     */
    public function getAddress(\WC_Order $order)
    {
        $address = new Address();
        $address->setType('delivery');

        if ($order->has_shipping_address()) {
            $address->setFirstName($order->get_shipping_first_name());
            $address->setLastName($order->get_shipping_last_name());
            $address->setStreet($order->get_shipping_address_1());
            $address->setAdditionStreet($order->get_shipping_address_2());
            $address->setZip($order->get_shipping_postcode());
            $address->setCountry($order->get_shipping_country());
            $address->setCity($order->get_shipping_city());
            $address->setCompany($order->get_shipping_company());
            $address->setState($order->get_shipping_state());
            $address->setEmail($order->get_meta('_shipping_email') ?: $order->get_meta('billing_email'));
            $address->setPhone($order->get_shipping_phone() ?: $order->get_billing_phone());
        } else {
            $address->setFirstName($order->get_billing_first_name());
            $address->setLastName($order->get_billing_last_name());
            $address->setStreet($order->get_billing_address_1());
            $address->setAdditionStreet($order->get_billing_address_2());
            $address->setZip($order->get_billing_postcode());
            $address->setCountry($order->get_billing_country());
            $address->setCity($order->get_billing_city());
            $address->setCompany($order->get_billing_company());
            $address->setState($order->get_billing_state());
            $address->setEmail($order->get_billing_email());
            $address->setPhone($order->get_billing_phone());
        }

        // Set customer identification only for Brazil or Taiwan
        if (in_array($address->getCountry(), ['BR', 'TW'])) {
            $customerIdentification = $order->get_meta('_customer_identification');
            if ($customerIdentification) {
                $address->setCustomerIdentification($customerIdentification);
            }
        }

        return $address;
    }

    /**
     * Request order quotes
     * @param \WC_Order $order
     * @return array
     * @throws ValidationException
     * @throws \Exception
     */
    public function getOrderQuotes(\WC_Order $order)
    {
        $orderQuote = CloudPrinterHelper::requestOrderQuotes($order);

        if (isset($orderQuote['error']['info'])) {
            throw new \Exception($orderQuote['error']['info']);
        }

        $shippingData = $order->get_meta('_cp_shipping_data');

        if (!$shippingData) {
            $wcSession = CloudPrinterHelper::getWCSession();
            $shippingData = $wcSession->get('cp_shipping_data');
            $order->update_meta_data('_cp_shipping_data', $shippingData);
        }

        $shippingData = $shippingData ? json_decode($shippingData, true) : [];

        $result = [];
        foreach ($orderQuote['shipments'] as $key => $shipment) {
            $quote = null;
            foreach ($shipment['quotes'] as $shippingQuote) {
                if ($shippingQuote['service'] == $shippingData[$key]['service'] && $shippingQuote['shipping_option'] == $shippingData[$key]['shipping_option']) {
                    $quote = $shippingQuote['quote'];
                    $shippingData[$key]['price'] = $shippingQuote['price'];
                    $shippingData[$key]['quote'] = $quote;
                    $shippingData[$key]['items'] = array_column($shipment['items'], 'reference');
                }
            }

            foreach ($shipment['items'] as $item) {
                $result[$item['reference']] = $quote;
            }
        }

        //Store shipping data to custom fields.
        update_post_meta($order->get_id(), '_cp_shipping_data', json_encode($shippingData));
        update_post_meta($order->get_id(), '_cp_order_total_product_price', $orderQuote['price']);

        return $result;
    }


    /**
     * Verify rest api request.
     * @return bool
     */
    public function verifyApiRequest()
    {
        $headers = array_change_key_case(getallheaders());

        return wp_verify_nonce($headers['x-wp-nonce'], 'wp_rest') && current_user_can( 'edit_posts' );
    }
}
