<?php


namespace CloudPrinterPlugin\Controllers;


use CloudPrinterPlugin\Helpers\CartHelper;
use CloudPrinterPlugin\Helpers\CloudPrinterHelper;
use CloudPrinterPlugin\Helpers\OrderHelper;
use CloudPrinterPlugin\Services\ConfigService;
use CloudPrinterPlugin\Services\ShippingService;

class ShippingController
{
    public function __construct()
    {
        // Apply taxes
        add_filter('cloudprinter_shipping_price', [$this, 'shippingPrice']);

        add_action('woocommerce_refund_created', [$this, 'handleShippingRefund'], 10, 2);

        // Override cart shipping tax
        add_filter('woocommerce_cart_get_taxes', [$this, 'calculateShippingCost'], 10, 2);

        // Apply shipping tax for tax total in order admin.
        add_filter('woocommerce_order_get_tax_totals', [$this, 'setShippingTaxesInAdmin'], 10, 2);
    }

    public function shippingPrice($price)
    {
        if (wc_tax_enabled() && 'incl' !== get_option( 'woocommerce_tax_display_shop' )) {

            if (!wc_prices_include_tax()) {
                $shippingService = new ShippingService();
                $taxes = $shippingService->calculateTax((float)$price);
                if (!empty($taxes)) {
                  //  $price += $taxes;
                }
            } else {
                $shippingService = new ShippingService();
                $price -= $shippingService->calculateTax((float)$price);
            }
        }

        return $price;
    }

    public function handleShippingRefund($refundId, $args)
    {
        if (!empty($args['line_items']['shipping'])) {
            $totalRefund = 0;

            $shippingRefund = $args['line_items']['shipping'];

            $currentRefundAmounts = [];
            if ($refundAmounts = get_post_meta($args['order_id'], '_shipping_refund_amount', true)) {
                $currentRefundAmounts = $refundAmounts;
            }

            $currentRefundTaxes = [];
            if ($refundTaxes = get_post_meta($args['order_id'], '_shipping_refund_tax', true)) {
                $currentRefundTaxes = $refundTaxes;
            }

            if (!empty($shippingRefund['refund_total'])) {
                $currentRefundAmounts[] = $shippingRefund['refund_total'];
                $totalRefund += $shippingRefund['refund_total'];
            }

            if (!empty($shippingRefund['refund_tax']['1'])) {
                $currentRefundTaxes[] = $shippingRefund['refund_tax']['1'];
                $totalRefund += $shippingRefund['refund_tax']['1'];
            }

            $orderIsShipped = get_post_meta($args['order_id'], '_cp_order_is_shipped', true);

            if (!$orderIsShipped) {
                // Total refund before order is shipped
                $preShippingRefundTotal = get_post_meta($args['order_id'], '_cp_pre_shipping_refund', true);

                if (!$preShippingRefundTotal) {
                    $preShippingRefundTotal = 0;
                }

                $preShippingRefundTotal += $totalRefund;
                update_post_meta($args['order_id'], '_cp_pre_shipping_refund', $preShippingRefundTotal);
            } else {
                // Total refund after order is shipped
                $postShippingRefundTotal = get_post_meta($args['order_id'], '_cp_post_shipping_refund', true);

                if (!$postShippingRefundTotal) {
                    $postShippingRefundTotal = 0;
                }

                $postShippingRefundTotal += $totalRefund;
                update_post_meta($args['order_id'], '_cp_post_shipping_refund', $postShippingRefundTotal);
            }

            update_post_meta($args['order_id'], '_shipping_refund_amount', $currentRefundAmounts);
            update_post_meta($args['order_id'], '_shipping_refund_tax', $currentRefundTaxes);
        }
    }

    /**
     * Override cart shipping tax
     */
    public function calculateShippingCost($taxes, $cart)
    {
        $configService = new ConfigService();

        if (!CartHelper::cartHasFreeShippingCoupon() && !$configService->isFreeShipping() && wc_tax_enabled()) {
            $selectedShipping = WC()->session->get('cp_shipping_data');
            $shippingService = new ShippingService();

            if (!empty($selectedShipping)) {
                $shippingLines = json_decode($selectedShipping, true);
                $shippingMarginPercentage = get_option('cp_shipping_price_margin') ?: 0;

                if (is_array($shippingLines)) {
                    foreach ($shippingLines as $shippingLine) {
                        $shippingLinePrice = isset($shippingLine['price']) ? $shippingLine['price'] : $shippingLine['native_price'];
                        if (!empty($shippingLinePrice)) {
                            $price = CloudPrinterHelper::calcPercentage($shippingLinePrice, $shippingMarginPercentage);
                            $tax = $shippingService->calculateTax($price);
                            $taxes[] = $tax;
                        }
                    }
                }
            }
        }

        return $taxes;
    }
    /**
     * Apply shipping tax for tax total in order admin.
     */
    public function setShippingTaxesInAdmin($taxes, $order)
    {
        $isFreeShipping = $order->get_meta('_cp_shipping_is_free') || OrderHelper::hasFreeShippingCoupon($order);
        if (!$isFreeShipping && wc_tax_enabled()) {
            $shippingData = $order->get_meta('_cp_shipping_data');

            $totalShippingTax = 0;
            if (!empty($shippingData)) {
                $shippingLines = json_decode($shippingData, true);
                $shippingService = new ShippingService();
                $shippingMarginPercentage = get_option('cp_shipping_price_margin') ?: 0;

                if (is_array($shippingLines)) {
                    foreach ($shippingLines as $shippingLine) {
                        $shippingLinePrice = isset($shippingLine['native_price']) ? $shippingLine['native_price'] : $shippingLine['price'];

                        if (!empty($shippingLinePrice)) {
                            $price = CloudPrinterHelper::calcPercentage($shippingLinePrice, $shippingMarginPercentage);
                            $totalShippingTax += $shippingService->calculateTax($price);
                        }
                    }
                }
            }

            foreach ($taxes as &$tax) {
                $tax->amount += $totalShippingTax;
                $tax->formatted_amount = wc_price($tax->amount);
            }
        }

        return $taxes;
    }
}
