<?php

namespace CloudPrinterPlugin\Services\CustomEmail;

/**
 * Class OrderShippedEmail
 *
 * @copyright 2020 by CloudPrinter
 * @author    Vasyl Kushniruk <vk@cloudprinter.com>
 */
class OrderShippedEmail extends \WC_Email {

    private $productsTrackingCodes = null;

    /**
     * Set email defaults
     *
     * @since 0.1
     */

    public function __construct() {

        // set ID, this simply needs to be a unique name
        $this->id = 'wc_cloudprinter_order';

        $this->customer_email = true;

        // this is the title in WooCommerce Email settings
        $this->title = 'Cloudprinter.com order shipped';

        // this is the description in WooCommerce email settings
        $this->description = 'Order Shipped Notification emails are sent when a ItemShipped signals come from cloudprinter.com';

        // these are the default heading and subject lines that can be overridden using the settings
        $this->heading = 'Order (#{order_number}) is shipped';
        $this->subject = 'Order (#{order_number}) is shipped';

        // these define the locations of the templates that this email should use, we'll just use the new order template since this email is similar
        $this->template_base = CP_ABS_PATH . 'src/views/emails/';
        $this->template_html  = 'order_shipped.php';

        $this->recipient = $this->get_option('recipient', get_option( 'admin_email' ));

        // Call parent constructor to load any other defaults not explicity defined here
        parent::__construct();

        // Trigger on new paid orders
        add_action('cloudprinter_order_is_shipped', [$this, 'trigger'], 10, 2);
    }

    /**
     * Determine if the email should actually be sent and setup email merge variables
     *
     * @since 0.1
     * @param \WC_Order $order
     * @param $emailShippingData
     */
    public function trigger( $order, $emailShippingData ) {
        if ( ! $order ) {
            return;
        }

        // setup order object
        $this->object = $order;
        $this->productsTrackingCodes = $this->getTrackingCodesLines($emailShippingData);

        $this->recipient = $this->object->get_billing_email();

        // replace variables in the subject/headings
        $this->find[] = '{order_date}';
        $this->replace[] = date_i18n( wc_date_format(), strtotime( $this->object->get_date_created() ) );

        $this->find[] = '{order_number}';
        $this->replace[] = $this->object->get_order_number();

        if ( ! $this->is_enabled() || ! $this->get_recipient() )
            return;

        //Send the email!
        $this->send( $this->get_recipient(), $this->get_subject(), $this->get_content(), $this->get_headers(), $this->get_attachments() );
    }

    private function getTrackingCodesLines($emailShippingData)
    {
        $trackingLines = [];
        $shipmentNumber = 1;
        foreach ($emailShippingData as $shippingOption => $shippingItems)
        {
            foreach ($shippingItems as $shippingItem) {
                $line = '';
                $trackingCode = !empty($shippingItem['tracking_code']) ? ' with tracking ' . $shippingItem['tracking_code'] : '';
                $line .= '<b>Shipment ' . $shipmentNumber . ': </b><br>' . $shippingOption . $trackingCode. '<br>';
                $line .=  join('<br>', $shippingItem['product_name']);

                if ($shippingItem['tracking_url']) {
                    $line .= '<br> <a href="' . $shippingItem['tracking_url'] . '" target="_blank">Click here for tracking information.</a>';
                }

                $trackingLines[] =  $line . '<br>';
                $shipmentNumber++;
            }
        }
        return $trackingLines;
    }

    /**
     * get_content_html function.
     *
     * @since 0.1
     * @return string
     */
    public function get_content_html() {
        ob_start();

        $productsTackingCodesText = join('<br>', $this->productsTrackingCodes);
        $template = $this->get_option('shipping_details_template');
        $text = str_replace('[%tracking-details-list%]', '<br><br>' . $productsTackingCodesText . '<br><br>', $template);


        extract([
            'order' => $this->object,
            'email_heading' => $this->get_heading(),
            'productsTackingCodes' => $this->productsTrackingCodes,
            'text' => $text
        ]);

        $emailTemplatePath = $this->template_base . $this->template_html;

        include apply_filters('cloudprinter_order_shipped_email_template_path', $emailTemplatePath);

        return ob_get_clean();
    }

    /**
     * get_content_plain function.
     *
     * @since 0.1
     * @return string
     */
    public function get_content_plain() {
        ob_start();
        woocommerce_get_template( $this->template_plain, array(
            'order'         => $this->object,
            'email_heading' => $this->get_heading()
        ) );
        return ob_get_clean();
    }


    /**
     * Initialize Settings Form Fields
     *
     * @since 2.0
     */
    public function init_form_fields() {

        $placeholder_text  = sprintf( __( 'Available placeholders: %s', 'woocommerce' ), '<code>' . esc_html( implode( '</code>, <code>', array_merge(array_keys( $this->placeholders ), ['{order_number}']) ) ) . '</code>' );
        $this->form_fields = array(
            'enabled'    => array(
                'title'   => 'Enable/Disable',
                'type'    => 'checkbox',
                'label'   => 'Enable this email notification',
                'default' => 'yes'
            ),
            'recipient'          => array(
                'title'       => __( 'Recipient(s)', 'woocommerce' ),
                'type'        => 'text',
                /* translators: %s: WP admin email */
                'description' => sprintf( __( 'Enter recipients (comma separated) for this email. Defaults to %s.', 'woocommerce' ), '<code>' . esc_attr( get_option( 'admin_email' ) ) . '</code>' ),
                'placeholder' => '',
                'default'     => '',
                'desc_tip'    => true,
            ),
            'subject'    => array(
                'title'       => 'Subject',
                'type'        => 'text',
                'description' => 'This controls the email subject line. ' . $placeholder_text,
                'placeholder' => '',
                'desc_tip'    => true,
                'default'     => $this->subject
            ),
            'heading'    => array(
                'title'       => 'Email Heading',
                'type'        => 'text',
                'description' => __( 'This controls the main heading contained within the email notification.') . $placeholder_text,
                'placeholder' => '',
                'default'     => $this->heading,
                'desc_tip'    => true,
            ),
            'shipping_details_template'    => array(
                'title'       => 'Shipping details template',
                'type'        => 'textarea',
                'description' =>  '
                    <p>Here you can create a template body for the email with shipping details that your clients will receive.</p>
                    <p>Use <i>[%tracking-details-list%]</i> to display a list of all tracking information for the different shipments in the order. <br>This looks as follows:
                        <ul style=" font-size: 11px;">
                            <li>
                                Shipment 1:<br>
                                [shipping_option] with tracking [tracking_code]<br>
                                [product_name]<br>
                                [tracking_information_link]
                            </li>
                            <li>
                                Shipment 2:<br>
                                [shipping_option] with tracking [tracking_code]<br>
                                [product_name]<br>
                                [tracking_information_link]
                            </li>
                            ----------
                            <li>
                                Shipment n:<br>
                                [shipping_option] with tracking [tracking_code]<br>
                                [product_name]<br>
                                [tracking_information_link]
                            </li>
                        </ul>
                    </p>
                ',
                'placeholder' => '',
                'default'     => ''.
                    "All items in your order are ready for shipment.\n" .
                    "[%tracking-details-list%]\n".
                    'Visit the courier’s website to track your package(s). Please keep in mind that it can take up to 24 hours for tracking information to become available.'
            ),

            'email_type' => array(
                'title'       => 'Email type',
                'type'        => 'select',
                'description' => 'Choose which format of email to send.',
                'default'     => 'html',
                'class'       => 'email_type',
                'options'     => array(
                    'plain'	    => __( 'Plain text', 'woocommerce' ),
                    'html' 	    => __( 'HTML', 'woocommerce' ),
                    'multipart' => __( 'Multipart', 'woocommerce' ),
                )
            )
        );
    }
}
